#include "Serialcommand.h"
#include "Arduino.h"

Serialcommand::Serialcommand()
{
  // 连接控制器
  Serial2.begin(115200);
  // clear serialport
  while (Serial2.read() >= 0)
  {
  }
  CmdState = 0;
  bufferIndex = 0;
  recv_flag = false;
  outBuffer[0] = 0xff;
  outBuffer[1] = 0xff;
}
Serialcommand::~Serialcommand()
{
}

void Serialcommand::Uart_Command_Rev(void)
{
  while (Serial2.available() > 0)
  {
    ch = Serial2.read();
    // printf("rev:%d\n", ch);
    switch (CmdState)
    {
      // 开始标志1
    case 0:
    {
      if (0xFF == ch)
      {
        CmdState = 1;
        // printf("st:%d\n", CmdState);
      }
    }
    break;
    // 开始标志2
    case 1:
    {
      if (0xFF == ch)
      {
        bufferIndex = 0;
        CmdState = 2;
        // printf("st:%d\n", CmdState);
      }
      else
        CmdState = 0;
    }
    break;
    // 取长度，准备收数据
    case 2:
    {
      datalen = ch;
      // printf("datalen:%d\n", datalen);
      if (datalen <= inbuffersize)
      {
        CmdState = 3;
        bufferIndex = 0;
        inBuffer[bufferIndex] = ch;
        // printf("buf:%d:%d\n", bufferIndex, inBuffer[bufferIndex]);
      }
      else // 超长
      {
        CmdState = 0;
      }
    }
    break;
    // 收数据直到结束
    case 3:
    {
      // printf("st3:%d:%d\n", bufferIndex, datalen);

      if (bufferIndex <= datalen)
      {
        bufferIndex++;
        inBuffer[bufferIndex] = ch;
        // printf("buf:%d:%d\n", bufferIndex, inBuffer[bufferIndex]);
        if (bufferIndex == datalen) // 收完了
        {
          CmdState = 0;
          recv_flag = true;
          bufferIndex = 0;
          return; // 不收数据了，先出去解析
        }
      }
    }
    break;
    }
  }
}
uint8_t Serialcommand::getsum(const uint8_t *buffer, size_t size)
{
  uint16_t usum = 0;
  uint8_t chksum = 0;
  for (uint8_t i = 0; i < size; i++)
  {
    usum += buffer[i];
  }
  chksum = (uint8_t)usum & 0xff; // 取低8位
  return chksum;
}
bool Serialcommand::checksum()
{
  uint8_t bufsum = inBuffer[datalen];             // 校验和
  uint8_t chksum = getsum(inBuffer, datalen - 1); // 不算sum字节
  if (chksum != bufsum)
  {
    printf("sum fault :data_sum:0x%02X,add_sum:0x%02X\n", bufsum, chksum);
  }
  return bufsum == chksum;
}

void Serialcommand::sendcmdret(uint8_t cmdret)
{
  // 长度
  outBuffer[2] = 0x02;
  // 标识命令返回
  outBuffer[3] = 0x01;
  // 返回数据
  outBuffer[4] = cmdret;
  outBuffer[5] = getsum(outBuffer + 2, 3);
  Serial2.write(outBuffer, 6);
}
void Serialcommand::getcommand()
{
  Uart_Command_Rev();
  // 收到有效数据
  if (recv_flag)
  {
    // printf("rev end len:%d\n", datalen);
    recv_flag = false;
    // 处理数据
    if (checksum())
    {
      uint8_t commandid = inBuffer[1];
      printf("rev ok cid:0x%02X\n", commandid);
      switch (commandid)
      {
      // 放钩
      case 0x01:
      {
        sendcmdret(0); // 成功
      }
      break;
      // 收钩
      case 0x02:
      {
        sendcmdret(0); // 成功
      }
      break;
      // 停止
      case 0x03:
      {
        sendcmdret(0); // 成功
      }
      break;
      // 查询当前状态
      case 0x04:
      {
        sendcmdret(0); //  发送状态
      }
      break;
      // 设置速度
      case 0x05:
      {
        sendcmdret(0); // 成功
      }
      break;
      default:
        sendcmdret(100); // 失败，未知命令
      }
    }
    memset(inBuffer, 0x00, sizeof(inBuffer));
  }
}
